import type React from "react";
import { useEffect, useState } from "react";

interface Feed {
  id: string;
  url: string;
  title?: string;
  description?: string;
  lastUpdated?: string;
  createdAt: string;
  active: boolean;
}

interface Stats {
  totalFeeds: number;
  activeFeeds: number;
  inactiveFeeds: number;
  totalEpisodes: number;
  pendingRequests: number;
  totalRequests: number;
  batchScheduler: {
    enabled: boolean;
    isRunning: boolean;
    canForceStop: boolean;
    lastRun?: string;
    nextRun?: string;
  };
  lastUpdated: string;
  adminPort: number;
  authEnabled: boolean;
}

interface EnvVars {
  [key: string]: string | undefined;
}

interface FeedRequest {
  id: string;
  url: string;
  requestedBy?: string;
  requestMessage?: string;
  status: "pending" | "approved" | "rejected";
  createdAt: string;
  reviewedAt?: string;
  reviewedBy?: string;
  adminNotes?: string;
}

interface Episode {
  id: string;
  title: string;
  description?: string;
  audioPath: string;
  duration?: number;
  fileSize?: number;
  createdAt: string;
  articleId: string;
  articleTitle: string;
  articleLink: string;
  articlePubDate: string;
  feedId: string;
  feedTitle?: string;
  feedUrl: string;
  feedCategory?: string;
}

interface Setting {
  key: string;
  value: string | null;
  isCredential: boolean;
  description: string;
  defaultValue: string;
  required: boolean;
  updatedAt: string;
}

interface CategoryData {
  feedCategories: string[];
  episodeCategories: string[];
  allCategories: string[];
}

interface CategoryCounts {
  feedCount: number;
  episodeCount: number;
}

function App() {
  const [feeds, setFeeds] = useState<Feed[]>([]);
  const [stats, setStats] = useState<Stats | null>(null);
  const [envVars, setEnvVars] = useState<EnvVars>({});
  const [settings, setSettings] = useState<Setting[]>([]);
  const [feedRequests, setFeedRequests] = useState<FeedRequest[]>([]);
  const [episodes, setEpisodes] = useState<Episode[]>([]);
  const [loading, setLoading] = useState(true);
  const [error, setError] = useState<string | null>(null);
  const [success, setSuccess] = useState<string | null>(null);
  const [newFeedUrl, setNewFeedUrl] = useState("");
  const [requestFilter, setRequestFilter] = useState<
    "all" | "pending" | "approved" | "rejected"
  >("all");
  const [approvalNotes, setApprovalNotes] = useState<{ [key: string]: string }>(
    {},
  );
  const [editingSettings, setEditingSettings] = useState<{
    [key: string]: string;
  }>({});
  const [categories, setCategories] = useState<CategoryData>({
    feedCategories: [],
    episodeCategories: [],
    allCategories: [],
  });
  const [categoryCounts, setCategoryCounts] = useState<{
    [category: string]: CategoryCounts;
  }>({});
  const [activeTab, setActiveTab] = useState<
    | "dashboard"
    | "feeds"
    | "episodes"
    | "env"
    | "settings"
    | "batch"
    | "requests"
    | "categories"
  >("dashboard");

  useEffect(() => {
    loadData();
  }, []);

  const loadData = async () => {
    setLoading(true);
    try {
      const [
        feedsRes,
        statsRes,
        envRes,
        settingsRes,
        requestsRes,
        episodesRes,
        categoriesRes,
      ] = await Promise.all([
        fetch("/api/admin/feeds"),
        fetch("/api/admin/stats"),
        fetch("/api/admin/env"),
        fetch("/api/admin/settings"),
        fetch("/api/admin/feed-requests"),
        fetch("/api/admin/episodes"),
        fetch("/api/admin/categories/all"),
      ]);

      if (
        !feedsRes.ok ||
        !statsRes.ok ||
        !envRes.ok ||
        !settingsRes.ok ||
        !requestsRes.ok ||
        !episodesRes.ok ||
        !categoriesRes.ok
      ) {
        throw new Error("Failed to load data");
      }

      const [
        feedsData,
        statsData,
        envData,
        settingsData,
        requestsData,
        episodesData,
        categoriesData,
      ] = await Promise.all([
        feedsRes.json(),
        statsRes.json(),
        envRes.json(),
        settingsRes.json(),
        requestsRes.json(),
        episodesRes.json(),
        categoriesRes.json(),
      ]);

      setFeeds(feedsData);
      setStats(statsData);
      setEnvVars(envData);
      setSettings(settingsData);
      setFeedRequests(requestsData);
      setEpisodes(episodesData);
      setCategories(categoriesData);

      // Load category counts for all categories
      const countsPromises = categoriesData.allCategories.map(
        async (category: string) => {
          const res = await fetch(
            `/api/admin/categories/${encodeURIComponent(category)}/counts`,
          );
          if (res.ok) {
            const counts = await res.json();
            return { category, counts };
          }
          return { category, counts: { feedCount: 0, episodeCount: 0 } };
        },
      );

      const countsResults = await Promise.all(countsPromises);
      const countsMap: { [category: string]: CategoryCounts } = {};
      countsResults.forEach(({ category, counts }) => {
        countsMap[category] = counts;
      });
      setCategoryCounts(countsMap);

      setError(null);
    } catch (err) {
      setError("データの読み込みに失敗しました");
      console.error("Error loading data:", err);
    } finally {
      setLoading(false);
    }
  };

  const addFeed = async (e: React.FormEvent) => {
    e.preventDefault();
    if (!newFeedUrl.trim()) return;

    try {
      const res = await fetch("/api/admin/feeds", {
        method: "POST",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify({ feedUrl: newFeedUrl }),
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        setNewFeedUrl("");
        loadData();
      } else {
        setError(data.error || "フィード追加に失敗しました");
      }
    } catch (err) {
      setError("フィード追加に失敗しました");
      console.error("Error adding feed:", err);
    }
  };

  const deleteFeed = async (feedId: string) => {
    if (
      !confirm(
        "本当にこのフィードを削除しますか？関連するすべての記事とエピソードも削除されます。",
      )
    ) {
      return;
    }

    try {
      const res = await fetch(`/api/admin/feeds/${feedId}`, {
        method: "DELETE",
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        loadData();
      } else {
        setError(data.error || "フィード削除に失敗しました");
      }
    } catch (err) {
      setError("フィード削除に失敗しました");
      console.error("Error deleting feed:", err);
    }
  };

  const toggleFeed = async (feedId: string, active: boolean) => {
    try {
      const res = await fetch(`/api/admin/feeds/${feedId}/toggle`, {
        method: "PATCH",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify({ active }),
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        loadData();
      } else {
        setError(data.error || "フィードステータス変更に失敗しました");
      }
    } catch (err) {
      setError("フィードステータス変更に失敗しました");
      console.error("Error toggling feed:", err);
    }
  };

  const triggerBatch = async () => {
    try {
      const res = await fetch("/api/admin/batch/trigger", {
        method: "POST",
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        loadData(); // Refresh data to update batch status
      } else {
        setError(data.error || "バッチ処理開始に失敗しました");
      }
    } catch (err) {
      setError("バッチ処理開始に失敗しました");
      console.error("Error triggering batch:", err);
    }
  };

  const forceStopBatch = async () => {
    if (
      !confirm(
        "実行中のバッチ処理を強制停止しますか？\n\n進行中の処理が中断され、データの整合性に影響が出る可能性があります。",
      )
    ) {
      return;
    }

    try {
      const res = await fetch("/api/admin/batch/force-stop", {
        method: "POST",
      });

      const data = await res.json();

      if (res.ok) {
        if (data.result === "STOPPED") {
          setSuccess(data.message);
        } else if (data.result === "NO_PROCESS") {
          setSuccess(data.message);
        }
        loadData(); // Refresh data to update batch status
      } else {
        setError(data.error || "バッチ処理強制停止に失敗しました");
      }
    } catch (err) {
      setError("バッチ処理強制停止に失敗しました");
      console.error("Error force stopping batch:", err);
    }
  };

  const toggleBatchScheduler = async (enable: boolean) => {
    try {
      const res = await fetch(
        `/api/admin/batch/${enable ? "enable" : "disable"}`,
        {
          method: "POST",
        },
      );

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        loadData(); // Refresh data to update batch status
      } else {
        setError(data.error || "バッチスケジューラーの状態変更に失敗しました");
      }
    } catch (err) {
      setError("バッチスケジューラーの状態変更に失敗しました");
      console.error("Error toggling batch scheduler:", err);
    }
  };

  const approveFeedRequest = async (requestId: string, notes: string) => {
    try {
      const res = await fetch(`/api/admin/feed-requests/${requestId}/approve`, {
        method: "PATCH",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify({ adminNotes: notes }),
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message || "フィードリクエストを承認しました");
        setApprovalNotes({ ...approvalNotes, [requestId]: "" });
        loadData();
      } else {
        setError(data.error || "フィードリクエストの承認に失敗しました");
      }
    } catch (err) {
      setError("フィードリクエストの承認に失敗しました");
      console.error("Error approving feed request:", err);
    }
  };

  const rejectFeedRequest = async (requestId: string, notes: string) => {
    if (!confirm("このフィードリクエストを拒否しますか？")) {
      return;
    }

    try {
      const res = await fetch(`/api/admin/feed-requests/${requestId}/reject`, {
        method: "PATCH",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify({ adminNotes: notes }),
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message || "フィードリクエストを拒否しました");
        setApprovalNotes({ ...approvalNotes, [requestId]: "" });
        loadData();
      } else {
        setError(data.error || "フィードリクエストの拒否に失敗しました");
      }
    } catch (err) {
      setError("フィードリクエストの拒否に失敗しました");
      console.error("Error rejecting feed request:", err);
    }
  };

  const updateApprovalNotes = (requestId: string, notes: string) => {
    setApprovalNotes({ ...approvalNotes, [requestId]: notes });
  };

  const updateSetting = async (key: string, value: string) => {
    try {
      const res = await fetch(`/api/admin/settings/${key}`, {
        method: "PUT",
        headers: { "Content-Type": "application/json" },
        body: JSON.stringify({ value }),
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        setEditingSettings({ ...editingSettings, [key]: "" });
        loadData();
      } else {
        setError(data.error || "設定更新に失敗しました");
      }
    } catch (err) {
      setError("設定更新に失敗しました");
      console.error("Error updating setting:", err);
    }
  };

  const startEditingSetting = (key: string, currentValue: string | null) => {
    setEditingSettings({ ...editingSettings, [key]: currentValue || "" });
  };

  const cancelEditingSetting = (key: string) => {
    const newEditing = { ...editingSettings };
    delete newEditing[key];
    setEditingSettings(newEditing);
  };

  const updateEditingValue = (key: string, value: string) => {
    setEditingSettings({ ...editingSettings, [key]: value });
  };

  const deleteCategory = async (
    category: string,
    target: "feeds" | "episodes" | "both",
  ) => {
    const targetText =
      target === "both"
        ? "フィードとエピソード"
        : target === "feeds"
          ? "フィード"
          : "エピソード";
    const counts = categoryCounts[category] || {
      feedCount: 0,
      episodeCount: 0,
    };
    const totalCount =
      target === "both"
        ? counts.feedCount + counts.episodeCount
        : target === "feeds"
          ? counts.feedCount
          : counts.episodeCount;

    if (
      !confirm(
        `本当にカテゴリ「${category}」を${targetText}から削除しますか？\n\n${totalCount}件のアイテムが影響を受けます。この操作は取り消せません。`,
      )
    ) {
      return;
    }

    try {
      const res = await fetch(
        `/api/admin/categories/${encodeURIComponent(category)}`,
        {
          method: "DELETE",
          headers: { "Content-Type": "application/json" },
          body: JSON.stringify({ target }),
        },
      );

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        loadData(); // Reload data to update category list and counts
      } else {
        setError(data.error || "カテゴリ削除に失敗しました");
      }
    } catch (err) {
      setError("カテゴリ削除に失敗しました");
      console.error("Error deleting category:", err);
    }
  };

  const deleteEpisode = async (episodeId: string, episodeTitle: string) => {
    if (
      !confirm(
        `本当にエピソード「${episodeTitle}」を削除しますか？\n\n音声ファイルも削除され、この操作は取り消せません。`,
      )
    ) {
      return;
    }

    try {
      const res = await fetch(`/api/admin/episodes/${episodeId}`, {
        method: "DELETE",
      });

      const data = await res.json();

      if (res.ok) {
        setSuccess(data.message);
        loadData();
      } else {
        setError(data.error || "エピソード削除に失敗しました");
      }
    } catch (err) {
      setError("エピソード削除に失敗しました");
      console.error("Error deleting episode:", err);
    }
  };

  const filteredRequests = feedRequests.filter((request) => {
    if (requestFilter === "all") return true;
    return request.status === requestFilter;
  });

  if (loading) {
    return (
      <div className="container">
        <div className="loading">読み込み中...</div>
      </div>
    );
  }

  return (
    <div className="container">
      <div className="header">
        <h1>管理者パネル</h1>
        <p className="subtitle">
          RSS Podcast Manager - 管理者用インターフェース
        </p>
      </div>

      {error && <div className="error">{error}</div>}
      {success && <div className="success">{success}</div>}

      <div className="card">
        <div className="card-header">
          <nav style={{ display: "flex", gap: "16px" }}>
            <button
              className={`btn ${activeTab === "dashboard" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("dashboard")}
            >
              ダッシュボード
            </button>
            <button
              className={`btn ${activeTab === "feeds" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("feeds")}
            >
              フィード管理
            </button>
            <button
              className={`btn ${activeTab === "episodes" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("episodes")}
            >
              エピソード管理
            </button>
            <button
              className={`btn ${activeTab === "batch" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("batch")}
            >
              バッチ管理
            </button>
            <button
              className={`btn ${activeTab === "requests" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("requests")}
            >
              フィード承認
            </button>
            <button
              className={`btn ${activeTab === "settings" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("settings")}
            >
              設定管理
            </button>
            <button
              className={`btn ${activeTab === "categories" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("categories")}
            >
              カテゴリ管理
            </button>
            <button
              className={`btn ${activeTab === "env" ? "btn-primary" : "btn-secondary"}`}
              onClick={() => setActiveTab("env")}
            >
              環境変数
            </button>
          </nav>
        </div>

        <div className="card-content">
          {activeTab === "dashboard" && (
            <>
              <div className="stats-grid">
                <div className="stat-card">
                  <div className="value">{stats?.totalFeeds || 0}</div>
                  <div className="label">総フィード数</div>
                </div>
                <div className="stat-card">
                  <div className="value">{stats?.activeFeeds || 0}</div>
                  <div className="label">アクティブフィード</div>
                </div>
                <div className="stat-card">
                  <div className="value">{stats?.inactiveFeeds || 0}</div>
                  <div className="label">非アクティブフィード</div>
                </div>
                <div className="stat-card">
                  <div className="value">{stats?.totalEpisodes || 0}</div>
                  <div className="label">総エピソード数</div>
                </div>
                <div className="stat-card">
                  <div className="value">{stats?.pendingRequests || 0}</div>
                  <div className="label">保留中リクエスト</div>
                </div>
                <div className="stat-card">
                  <div
                    className="value"
                    style={{
                      color: stats?.batchScheduler?.enabled
                        ? "#28a745"
                        : "#dc3545",
                    }}
                  >
                    {stats?.batchScheduler?.enabled ? "ON" : "OFF"}
                  </div>
                  <div className="label">バッチスケジューラー</div>
                </div>
              </div>

              <div style={{ marginBottom: "20px" }}>
                <button
                  className="btn btn-success"
                  onClick={triggerBatch}
                  disabled={stats?.batchScheduler?.isRunning}
                >
                  {stats?.batchScheduler?.isRunning
                    ? "バッチ処理実行中..."
                    : "バッチ処理を手動実行"}
                </button>
                {stats?.batchScheduler?.canForceStop && (
                  <button
                    className="btn btn-danger"
                    onClick={forceStopBatch}
                    style={{ marginLeft: "8px" }}
                  >
                    強制停止
                  </button>
                )}
                <button
                  className="btn btn-primary"
                  onClick={loadData}
                  style={{ marginLeft: "8px" }}
                >
                  データを再読み込み
                </button>
              </div>

              <div style={{ fontSize: "14px", color: "#7f8c8d" }}>
                <p>管理者パネルポート: {stats?.adminPort}</p>
                <p>認証: {stats?.authEnabled ? "有効" : "無効"}</p>
                <p>
                  最終更新:{" "}
                  {stats?.lastUpdated
                    ? new Date(stats.lastUpdated).toLocaleString("ja-JP")
                    : "不明"}
                </p>
              </div>
            </>
          )}

          {activeTab === "feeds" && (
            <>
              <form onSubmit={addFeed} className="add-feed-form">
                <div className="form-group">
                  <label htmlFor="feedUrl">新しいフィードURL</label>
                  <input
                    id="feedUrl"
                    type="url"
                    className="input"
                    value={newFeedUrl}
                    onChange={(e) => setNewFeedUrl(e.target.value)}
                    placeholder="https://example.com/feed.xml"
                    required
                  />
                </div>
                <button type="submit" className="btn btn-success">
                  追加
                </button>
              </form>

              <div style={{ marginTop: "24px" }}>
                <h3>フィード一覧 ({feeds.length}件)</h3>
                {feeds.length === 0 ? (
                  <p
                    style={{
                      color: "#7f8c8d",
                      textAlign: "center",
                      padding: "20px",
                    }}
                  >
                    フィードが登録されていません
                  </p>
                ) : (
                  <ul className="feeds-list">
                    {feeds.map((feed) => (
                      <li key={feed.id} className="feed-item">
                        <div className="feed-info">
                          <h3>{feed.title || "タイトル未設定"}</h3>
                          <div className="url">{feed.url}</div>
                          <span
                            className={`status ${feed.active ? "active" : "inactive"}`}
                          >
                            {feed.active ? "アクティブ" : "非アクティブ"}
                          </span>
                        </div>
                        <div className="feed-actions">
                          <button
                            className={`btn ${feed.active ? "btn-warning" : "btn-success"}`}
                            onClick={() => toggleFeed(feed.id, !feed.active)}
                          >
                            {feed.active ? "無効化" : "有効化"}
                          </button>
                          <button
                            className="btn btn-danger"
                            onClick={() => deleteFeed(feed.id)}
                          >
                            削除
                          </button>
                        </div>
                      </li>
                    ))}
                  </ul>
                )}
              </div>
            </>
          )}

          {activeTab === "episodes" && (
            <>
              <h3>エピソード管理</h3>
              <p style={{ marginBottom: "20px", color: "#7f8c8d" }}>
                生成済みのエピソードを確認・削除できます。
              </p>

              <div style={{ marginBottom: "20px" }}>
                <div
                  className="stats-grid"
                  style={{ gridTemplateColumns: "repeat(3, 1fr)" }}
                >
                  <div className="stat-card">
                    <div className="value">{episodes.length}</div>
                    <div className="label">総エピソード数</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {
                        episodes
                          .filter((ep) => ep.feedCategory)
                          .map((ep) => ep.feedCategory)
                          .filter(
                            (category, index, arr) =>
                              arr.indexOf(category) === index,
                          ).length
                      }
                    </div>
                    <div className="label">カテゴリー数</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {episodes.reduce(
                        (acc, ep) => acc + (ep.fileSize || 0),
                        0,
                      ) /
                        (1024 * 1024) >
                      1
                        ? `${Math.round(episodes.reduce((acc, ep) => acc + (ep.fileSize || 0), 0) / (1024 * 1024))}MB`
                        : `${Math.round(episodes.reduce((acc, ep) => acc + (ep.fileSize || 0), 0) / 1024)}KB`}
                    </div>
                    <div className="label">合計ファイルサイズ</div>
                  </div>
                </div>
              </div>

              {episodes.length === 0 ? (
                <p
                  style={{
                    color: "#7f8c8d",
                    textAlign: "center",
                    padding: "20px",
                  }}
                >
                  エピソードがありません
                </p>
              ) : (
                <div style={{ marginTop: "24px" }}>
                  <h4>エピソード一覧 ({episodes.length}件)</h4>
                  <ul className="feeds-list">
                    {episodes.map((episode) => (
                      <li key={episode.id} className="feed-item">
                        <div className="feed-info">
                          <h3>{episode.title}</h3>
                          <div
                            className="url"
                            style={{ fontSize: "14px", color: "#666" }}
                          >
                            フィード: {episode.feedTitle || episode.feedUrl}
                            {episode.feedCategory && (
                              <span
                                style={{
                                  marginLeft: "8px",
                                  padding: "2px 6px",
                                  background: "#e9ecef",
                                  borderRadius: "4px",
                                  fontSize: "12px",
                                }}
                              >
                                {episode.feedCategory}
                              </span>
                            )}
                          </div>
                          <div
                            className="url"
                            style={{ fontSize: "14px", color: "#666" }}
                          >
                            記事:{" "}
                            <a
                              href={episode.articleLink}
                              target="_blank"
                              rel="noopener noreferrer"
                            >
                              {episode.articleTitle}
                            </a>
                          </div>
                          {episode.description && (
                            <div
                              style={{
                                fontSize: "14px",
                                color: "#777",
                                marginTop: "4px",
                              }}
                            >
                              {episode.description.length > 100
                                ? episode.description.substring(0, 100) + "..."
                                : episode.description}
                            </div>
                          )}
                          <div
                            style={{
                              fontSize: "12px",
                              color: "#999",
                              marginTop: "8px",
                            }}
                          >
                            <span>
                              作成日:{" "}
                              {new Date(episode.createdAt).toLocaleString(
                                "ja-JP",
                              )}
                            </span>
                            {episode.duration && (
                              <>
                                <span style={{ margin: "0 8px" }}>|</span>
                                <span>
                                  再生時間: {Math.round(episode.duration / 60)}
                                  分
                                </span>
                              </>
                            )}
                            {episode.fileSize && (
                              <>
                                <span style={{ margin: "0 8px" }}>|</span>
                                <span>
                                  ファイルサイズ:{" "}
                                  {episode.fileSize > 1024 * 1024
                                    ? `${Math.round(episode.fileSize / (1024 * 1024))}MB`
                                    : `${Math.round(episode.fileSize / 1024)}KB`}
                                </span>
                              </>
                            )}
                          </div>
                          <div style={{ marginTop: "8px" }}>
                            <a
                              href={episode.audioPath}
                              target="_blank"
                              rel="noopener noreferrer"
                              style={{
                                fontSize: "12px",
                                color: "#007bff",
                                textDecoration: "none",
                              }}
                            >
                              🎵 音声ファイルを再生
                            </a>
                          </div>
                        </div>
                        <div className="feed-actions">
                          <button
                            className="btn btn-danger"
                            onClick={() =>
                              deleteEpisode(episode.id, episode.title)
                            }
                          >
                            削除
                          </button>
                        </div>
                      </li>
                    ))}
                  </ul>
                </div>
              )}
            </>
          )}

          {activeTab === "batch" && (
            <>
              <h3>バッチ処理管理</h3>
              <p style={{ marginBottom: "20px", color: "#7f8c8d" }}>
                定期バッチ処理の状態管理と手動実行を行えます。
              </p>

              <div className="stats-grid" style={{ marginBottom: "24px" }}>
                <div className="stat-card">
                  <div
                    className="value"
                    style={{
                      color: stats?.batchScheduler?.enabled
                        ? "#28a745"
                        : "#dc3545",
                    }}
                  >
                    {stats?.batchScheduler?.enabled ? "有効" : "無効"}
                  </div>
                  <div className="label">スケジューラー状態</div>
                </div>
                <div className="stat-card">
                  <div
                    className="value"
                    style={{
                      color: stats?.batchScheduler?.isRunning
                        ? "#ffc107"
                        : "#6c757d",
                    }}
                  >
                    {stats?.batchScheduler?.isRunning ? "実行中" : "待機中"}
                  </div>
                  <div className="label">実行状態</div>
                </div>
                <div className="stat-card">
                  <div className="value" style={{ fontSize: "12px" }}>
                    {stats?.batchScheduler?.lastRun
                      ? new Date(stats.batchScheduler.lastRun).toLocaleString(
                          "ja-JP",
                        )
                      : "未実行"}
                  </div>
                  <div className="label">前回実行</div>
                </div>
                <div className="stat-card">
                  <div className="value" style={{ fontSize: "12px" }}>
                    {stats?.batchScheduler?.nextRun
                      ? new Date(stats.batchScheduler.nextRun).toLocaleString(
                          "ja-JP",
                        )
                      : "未予定"}
                  </div>
                  <div className="label">次回実行</div>
                </div>
              </div>

              <div style={{ marginBottom: "24px" }}>
                <h4>スケジューラー制御</h4>
                <div
                  style={{ display: "flex", gap: "12px", marginTop: "12px" }}
                >
                  <button
                    className="btn btn-success"
                    onClick={() => toggleBatchScheduler(true)}
                    disabled={stats?.batchScheduler?.enabled}
                  >
                    スケジューラーを有効化
                  </button>
                  <button
                    className="btn btn-warning"
                    onClick={() => toggleBatchScheduler(false)}
                    disabled={!stats?.batchScheduler?.enabled}
                  >
                    スケジューラーを無効化
                  </button>
                </div>
                <p
                  style={{
                    fontSize: "14px",
                    color: "#6c757d",
                    marginTop: "8px",
                  }}
                >
                  スケジューラーを無効化すると、定期的なバッチ処理が停止します。手動実行は引き続き可能です。
                </p>
              </div>

              <div style={{ marginBottom: "24px" }}>
                <h4>手動実行</h4>
                <div
                  style={{ display: "flex", gap: "12px", marginTop: "12px" }}
                >
                  <button
                    className="btn btn-primary"
                    onClick={triggerBatch}
                    disabled={stats?.batchScheduler?.isRunning}
                  >
                    {stats?.batchScheduler?.isRunning
                      ? "バッチ処理実行中..."
                      : "バッチ処理を手動実行"}
                  </button>
                  {stats?.batchScheduler?.canForceStop && (
                    <button className="btn btn-danger" onClick={forceStopBatch}>
                      強制停止
                    </button>
                  )}
                </div>
                <p
                  style={{
                    fontSize: "14px",
                    color: "#6c757d",
                    marginTop: "8px",
                  }}
                >
                  スケジューラーの状態に関係なく、バッチ処理を手動で実行できます。実行中の場合は強制停止も可能です。
                </p>
              </div>

              <div
                style={{
                  padding: "16px",
                  background: "#f8f9fa",
                  borderRadius: "4px",
                }}
              >
                <h4>バッチ処理について</h4>
                <ul
                  style={{
                    fontSize: "14px",
                    color: "#6c757d",
                    marginTop: "8px",
                    paddingLeft: "20px",
                  }}
                >
                  <li>定期バッチ処理は6時間ごとに実行されます</li>
                  <li>新しいRSS記事の取得、要約生成、音声合成を行います</li>
                  <li>
                    スケジューラーが無効の場合、定期実行は停止しますが手動実行は可能です
                  </li>
                  <li>
                    バッチ処理の実行中は重複実行を防ぐため、新たな実行はスキップされます
                  </li>
                  <li>
                    <strong>強制停止:</strong>{" "}
                    実行中のバッチ処理を緊急停止できます（データ整合性に注意）
                  </li>
                </ul>
              </div>
            </>
          )}

          {activeTab === "requests" && (
            <>
              <h3>フィードリクエスト管理</h3>
              <p style={{ marginBottom: "20px", color: "#7f8c8d" }}>
                ユーザーから送信されたフィード追加リクエストを承認・拒否できます。
              </p>

              <div style={{ marginBottom: "20px" }}>
                <div style={{ display: "flex", gap: "8px" }}>
                  <button
                    className={`btn ${requestFilter === "all" ? "btn-primary" : "btn-secondary"}`}
                    onClick={() => setRequestFilter("all")}
                  >
                    すべて ({feedRequests.length})
                  </button>
                  <button
                    className={`btn ${requestFilter === "pending" ? "btn-primary" : "btn-secondary"}`}
                    onClick={() => setRequestFilter("pending")}
                  >
                    保留中 (
                    {feedRequests.filter((r) => r.status === "pending").length})
                  </button>
                  <button
                    className={`btn ${requestFilter === "approved" ? "btn-primary" : "btn-secondary"}`}
                    onClick={() => setRequestFilter("approved")}
                  >
                    承認済み (
                    {feedRequests.filter((r) => r.status === "approved").length}
                    )
                  </button>
                  <button
                    className={`btn ${requestFilter === "rejected" ? "btn-primary" : "btn-secondary"}`}
                    onClick={() => setRequestFilter("rejected")}
                  >
                    拒否済み (
                    {feedRequests.filter((r) => r.status === "rejected").length}
                    )
                  </button>
                </div>
              </div>

              {filteredRequests.length === 0 ? (
                <p
                  style={{
                    color: "#7f8c8d",
                    textAlign: "center",
                    padding: "20px",
                  }}
                >
                  {requestFilter === "all"
                    ? "フィードリクエストがありません"
                    : `${requestFilter === "pending" ? "保留中" : requestFilter === "approved" ? "承認済み" : "拒否済み"}のリクエストがありません`}
                </p>
              ) : (
                <div className="requests-list">
                  {filteredRequests.map((request) => (
                    <div
                      key={request.id}
                      className="feed-item"
                      style={{ marginBottom: "16px" }}
                    >
                      <div className="feed-info">
                        <h4 style={{ margin: "0 0 8px 0", fontSize: "16px" }}>
                          {request.url}
                        </h4>
                        {request.requestMessage && (
                          <p
                            style={{
                              margin: "0 0 8px 0",
                              fontSize: "14px",
                              color: "#666",
                            }}
                          >
                            メッセージ: {request.requestMessage}
                          </p>
                        )}
                        <div style={{ fontSize: "12px", color: "#999" }}>
                          <span>申請者: {request.requestedBy || "匿名"}</span>
                          <span style={{ margin: "0 8px" }}>|</span>
                          <span>
                            申請日:{" "}
                            {new Date(request.createdAt).toLocaleString(
                              "ja-JP",
                            )}
                          </span>
                          {request.reviewedAt && (
                            <>
                              <span style={{ margin: "0 8px" }}>|</span>
                              <span>
                                審査日:{" "}
                                {new Date(request.reviewedAt).toLocaleString(
                                  "ja-JP",
                                )}
                              </span>
                            </>
                          )}
                        </div>
                        <span
                          className={`status ${request.status === "approved" ? "active" : request.status === "rejected" ? "inactive" : ""}`}
                        >
                          {request.status === "pending"
                            ? "保留中"
                            : request.status === "approved"
                              ? "承認済み"
                              : "拒否済み"}
                        </span>
                        {request.adminNotes && (
                          <div
                            style={{
                              marginTop: "8px",
                              padding: "8px",
                              background: "#f8f9fa",
                              borderRadius: "4px",
                              fontSize: "14px",
                            }}
                          >
                            <strong>管理者メモ:</strong> {request.adminNotes}
                          </div>
                        )}
                      </div>
                      {request.status === "pending" && (
                        <div
                          className="feed-actions"
                          style={{
                            flexDirection: "column",
                            gap: "8px",
                            minWidth: "200px",
                          }}
                        >
                          <textarea
                            placeholder="管理者メモ（任意）"
                            value={approvalNotes[request.id] || ""}
                            onChange={(e) =>
                              updateApprovalNotes(request.id, e.target.value)
                            }
                            style={{
                              width: "100%",
                              minHeight: "60px",
                              padding: "8px",
                              border: "1px solid #ddd",
                              borderRadius: "4px",
                              fontSize: "12px",
                              resize: "vertical",
                            }}
                          />
                          <div style={{ display: "flex", gap: "4px" }}>
                            <button
                              className="btn btn-success"
                              onClick={() =>
                                approveFeedRequest(
                                  request.id,
                                  approvalNotes[request.id] || "",
                                )
                              }
                              style={{ fontSize: "12px", padding: "6px 12px" }}
                            >
                              承認
                            </button>
                            <button
                              className="btn btn-danger"
                              onClick={() =>
                                rejectFeedRequest(
                                  request.id,
                                  approvalNotes[request.id] || "",
                                )
                              }
                              style={{ fontSize: "12px", padding: "6px 12px" }}
                            >
                              拒否
                            </button>
                          </div>
                        </div>
                      )}
                    </div>
                  ))}
                </div>
              )}
            </>
          )}

          {activeTab === "settings" && (
            <>
              <h3>設定管理</h3>
              <p style={{ marginBottom: "20px", color: "#7f8c8d" }}>
                システム設定を管理できます。設定はデータベースに保存され、アプリケーション全体に適用されます。
              </p>

              <div style={{ marginBottom: "20px" }}>
                <div
                  className="stats-grid"
                  style={{ gridTemplateColumns: "repeat(4, 1fr)" }}
                >
                  <div className="stat-card">
                    <div className="value">{settings.length}</div>
                    <div className="label">総設定数</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {
                        settings.filter(
                          (s) => s.value !== null && s.value !== "",
                        ).length
                      }
                    </div>
                    <div className="label">設定済み</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {settings.filter((s) => s.required).length}
                    </div>
                    <div className="label">必須設定</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {settings.filter((s) => s.isCredential).length}
                    </div>
                    <div className="label">認証情報</div>
                  </div>
                </div>
              </div>

              <div className="settings-list">
                {settings.map((setting) => (
                  <div
                    key={setting.key}
                    className="setting-item"
                    style={{
                      display: "flex",
                      alignItems: "center",
                      padding: "16px",
                      border: "1px solid #ddd",
                      borderRadius: "4px",
                      marginBottom: "12px",
                      backgroundColor:
                        setting.required &&
                        (!setting.value || setting.value === "")
                          ? "#fff5f5"
                          : "#fff",
                    }}
                  >
                    <div style={{ flex: 1 }}>
                      <div
                        style={{
                          display: "flex",
                          alignItems: "center",
                          gap: "8px",
                          marginBottom: "4px",
                        }}
                      >
                        <h4 style={{ margin: 0, fontSize: "16px" }}>
                          {setting.key}
                        </h4>
                        {setting.required && (
                          <span
                            style={{
                              padding: "2px 6px",
                              background: "#dc3545",
                              color: "white",
                              borderRadius: "4px",
                              fontSize: "10px",
                            }}
                          >
                            必須
                          </span>
                        )}
                        {setting.isCredential && (
                          <span
                            style={{
                              padding: "2px 6px",
                              background: "#ffc107",
                              color: "black",
                              borderRadius: "4px",
                              fontSize: "10px",
                            }}
                          >
                            認証情報
                          </span>
                        )}
                      </div>
                      <p
                        style={{
                          margin: "0 0 8px 0",
                          fontSize: "14px",
                          color: "#666",
                        }}
                      >
                        {setting.description}
                      </p>
                      <div style={{ fontSize: "12px", color: "#999" }}>
                        デフォルト値: {setting.defaultValue || "なし"} |
                        最終更新:{" "}
                        {new Date(setting.updatedAt).toLocaleString("ja-JP")}
                      </div>

                      {editingSettings[setting.key] !== undefined ? (
                        <div
                          style={{
                            marginTop: "8px",
                            display: "flex",
                            gap: "8px",
                            alignItems: "center",
                          }}
                        >
                          <input
                            type={setting.isCredential ? "password" : "text"}
                            value={editingSettings[setting.key]}
                            onChange={(e) =>
                              updateEditingValue(setting.key, e.target.value)
                            }
                            placeholder={setting.defaultValue || "値を入力..."}
                            style={{
                              flex: 1,
                              padding: "6px 10px",
                              border: "1px solid #ddd",
                              borderRadius: "4px",
                              fontSize: "14px",
                            }}
                          />
                          <button
                            className="btn btn-success"
                            onClick={() =>
                              updateSetting(
                                setting.key,
                                editingSettings[setting.key],
                              )
                            }
                            style={{ fontSize: "12px", padding: "6px 12px" }}
                          >
                            保存
                          </button>
                          <button
                            className="btn btn-secondary"
                            onClick={() => cancelEditingSetting(setting.key)}
                            style={{ fontSize: "12px", padding: "6px 12px" }}
                          >
                            キャンセル
                          </button>
                        </div>
                      ) : (
                        <div
                          style={{
                            marginTop: "8px",
                            display: "flex",
                            alignItems: "center",
                            gap: "12px",
                          }}
                        >
                          <div
                            style={{
                              flex: 1,
                              padding: "6px 10px",
                              background: "#f8f9fa",
                              border: "1px solid #e9ecef",
                              borderRadius: "4px",
                              fontSize: "14px",
                              minHeight: "32px",
                              display: "flex",
                              alignItems: "center",
                            }}
                          >
                            {setting.value === null || setting.value === "" ? (
                              <span
                                style={{
                                  color: "#dc3545",
                                  fontStyle: "italic",
                                }}
                              >
                                未設定
                              </span>
                            ) : setting.isCredential ? (
                              <span style={{ color: "#666" }}>••••••••</span>
                            ) : (
                              setting.value
                            )}
                          </div>
                          <button
                            className="btn btn-primary"
                            onClick={() =>
                              startEditingSetting(setting.key, setting.value)
                            }
                            style={{ fontSize: "12px", padding: "6px 12px" }}
                          >
                            編集
                          </button>
                        </div>
                      )}
                    </div>
                  </div>
                ))}
              </div>

              <div
                style={{
                  marginTop: "20px",
                  padding: "16px",
                  background: "#f8f9fa",
                  borderRadius: "4px",
                }}
              >
                <h4>設定について</h4>
                <ul
                  style={{
                    fontSize: "14px",
                    color: "#6c757d",
                    marginTop: "8px",
                    paddingLeft: "20px",
                  }}
                >
                  <li>
                    <strong>必須設定:</strong>{" "}
                    アプリケーションの動作に必要な設定です
                  </li>
                  <li>
                    <strong>認証情報:</strong>{" "}
                    セキュリティ上重要な情報で、表示時にマスクされます
                  </li>
                  <li>設定の変更は即座にアプリケーションに反映されます</li>
                  <li>
                    デフォルト値が設定されている項目は、空にするとデフォルト値が使用されます
                  </li>
                </ul>
              </div>
            </>
          )}

          {activeTab === "categories" && (
            <>
              <h3>カテゴリ管理</h3>
              <p style={{ marginBottom: "20px", color: "#7f8c8d" }}>
                フィードとエピソードのカテゴリを管理できます。不要なカテゴリを削除してデータベースをクリーンアップできます。
              </p>

              <div style={{ marginBottom: "20px" }}>
                <div
                  className="stats-grid"
                  style={{ gridTemplateColumns: "repeat(3, 1fr)" }}
                >
                  <div className="stat-card">
                    <div className="value">
                      {categories.allCategories.length}
                    </div>
                    <div className="label">総カテゴリ数</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {categories.feedCategories.length}
                    </div>
                    <div className="label">フィードカテゴリ</div>
                  </div>
                  <div className="stat-card">
                    <div className="value">
                      {categories.episodeCategories.length}
                    </div>
                    <div className="label">エピソードカテゴリ</div>
                  </div>
                </div>
              </div>

              {categories.allCategories.length === 0 ? (
                <p
                  style={{
                    color: "#7f8c8d",
                    textAlign: "center",
                    padding: "20px",
                  }}
                >
                  カテゴリがありません
                </p>
              ) : (
                <div style={{ marginTop: "24px" }}>
                  <h4>カテゴリ一覧 ({categories.allCategories.length}件)</h4>
                  <div
                    style={{
                      marginBottom: "16px",
                      fontSize: "14px",
                      color: "#666",
                    }}
                  >
                    削除対象を選択してから削除ボタンをクリックしてください。
                  </div>

                  <div className="category-list">
                    {categories.allCategories.map((category) => {
                      const counts = categoryCounts[category] || {
                        feedCount: 0,
                        episodeCount: 0,
                      };
                      const isInFeeds =
                        categories.feedCategories.includes(category);
                      const isInEpisodes =
                        categories.episodeCategories.includes(category);

                      return (
                        <div
                          key={category}
                          className="feed-item"
                          style={{ marginBottom: "16px" }}
                        >
                          <div className="feed-info">
                            <h4
                              style={{ margin: "0 0 8px 0", fontSize: "16px" }}
                            >
                              {category}
                            </h4>
                            <div
                              style={{
                                fontSize: "14px",
                                color: "#666",
                                marginBottom: "8px",
                              }}
                            >
                              <span>フィード: {counts.feedCount}件</span>
                              <span style={{ margin: "0 12px" }}>|</span>
                              <span>エピソード: {counts.episodeCount}件</span>
                              <span style={{ margin: "0 12px" }}>|</span>
                              <span>
                                合計: {counts.feedCount + counts.episodeCount}件
                              </span>
                            </div>
                            <div style={{ fontSize: "12px", color: "#999" }}>
                              <span
                                style={{
                                  padding: "2px 6px",
                                  background: isInFeeds ? "#e3f2fd" : "#f5f5f5",
                                  color: isInFeeds ? "#1976d2" : "#999",
                                  borderRadius: "4px",
                                  marginRight: "8px",
                                }}
                              >
                                フィード: {isInFeeds ? "使用中" : "未使用"}
                              </span>
                              <span
                                style={{
                                  padding: "2px 6px",
                                  background: isInEpisodes
                                    ? "#e8f5e8"
                                    : "#f5f5f5",
                                  color: isInEpisodes ? "#388e3c" : "#999",
                                  borderRadius: "4px",
                                }}
                              >
                                エピソード: {isInEpisodes ? "使用中" : "未使用"}
                              </span>
                            </div>
                          </div>
                          <div
                            className="feed-actions"
                            style={{
                              flexDirection: "column",
                              gap: "8px",
                              minWidth: "160px",
                            }}
                          >
                            {isInFeeds && (
                              <button
                                className="btn btn-warning"
                                onClick={() =>
                                  deleteCategory(category, "feeds")
                                }
                                style={{
                                  fontSize: "12px",
                                  padding: "6px 12px",
                                }}
                              >
                                フィードから削除
                              </button>
                            )}
                            {isInEpisodes && (
                              <button
                                className="btn btn-warning"
                                onClick={() =>
                                  deleteCategory(category, "episodes")
                                }
                                style={{
                                  fontSize: "12px",
                                  padding: "6px 12px",
                                }}
                              >
                                エピソードから削除
                              </button>
                            )}
                            {(isInFeeds || isInEpisodes) && (
                              <button
                                className="btn btn-danger"
                                onClick={() => deleteCategory(category, "both")}
                                style={{
                                  fontSize: "12px",
                                  padding: "6px 12px",
                                }}
                              >
                                すべてから削除
                              </button>
                            )}
                          </div>
                        </div>
                      );
                    })}
                  </div>
                </div>
              )}

              <div
                style={{
                  marginTop: "20px",
                  padding: "16px",
                  background: "#f8f9fa",
                  borderRadius: "4px",
                }}
              >
                <h4>カテゴリ削除について</h4>
                <ul
                  style={{
                    fontSize: "14px",
                    color: "#6c757d",
                    marginTop: "8px",
                    paddingLeft: "20px",
                  }}
                >
                  <li>
                    <strong>フィードから削除:</strong>{" "}
                    フィードのカテゴリのみを削除します
                  </li>
                  <li>
                    <strong>エピソードから削除:</strong>{" "}
                    エピソードのカテゴリのみを削除します
                  </li>
                  <li>
                    <strong>すべてから削除:</strong>{" "}
                    フィードとエピソード両方からカテゴリを削除します
                  </li>
                  <li>
                    削除されたカテゴリは NULL に設定され、分類が解除されます
                  </li>
                  <li>この操作は元に戻すことができません</li>
                </ul>
              </div>
            </>
          )}

          {activeTab === "env" && (
            <>
              <h3>環境変数設定</h3>
              <p style={{ marginBottom: "20px", color: "#7f8c8d" }}>
                現在の環境変数設定を表示しています。機密情報は***SET***と表示されます。
              </p>

              <ul className="env-list">
                {Object.entries(envVars).map(([key, value]) => (
                  <li key={key} className="env-item">
                    <div className="env-key">{key}</div>
                    <div className="env-value">
                      {value === undefined ? "未設定" : value}
                    </div>
                  </li>
                ))}
              </ul>

              <div
                style={{
                  marginTop: "20px",
                  padding: "16px",
                  background: "#f8f9fa",
                  borderRadius: "4px",
                }}
              >
                <h4>環境変数の設定方法</h4>
                <p
                  style={{
                    fontSize: "14px",
                    color: "#6c757d",
                    marginTop: "8px",
                  }}
                >
                  環境変数を変更するには、.envファイルを編集するか、システムの環境変数を設定してください。
                  変更後はサーバーの再起動が必要です。
                </p>
              </div>
            </>
          )}
        </div>
      </div>
    </div>
  );
}

export default App;
